/*
 * Quoridor server and related libraries
 * Copyright (C) 2009-2010 Jaroslav Tulach <jaroslav.tulach@apidesign.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. Look for COPYING file in the top folder.
 * If not, see http://www.gnu.org/licenses/.
 */

package cz.xelfi.quoridor.freemarkerdor;

import cz.xelfi.quoridor.Board;
import cz.xelfi.quoridor.Fence;
import cz.xelfi.quoridor.Player;
import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Rectangle;
import java.awt.image.BufferedImage;

/** Convertor of a board to Image.
 *
 * @author Jaroslav Tulach <jtulach@netbeans.org>
 */
final class BoardImage  {
    static Image draw(Board b, int fieldSize) {
        int fifth = fieldSize / 10;

        BufferedImage img = new BufferedImage(fieldSize * 9, fieldSize * 9, BufferedImage.TYPE_INT_ARGB);
        Graphics2D g = (Graphics2D) img.getGraphics();

        g.setColor(Color.lightGray);
        for (int i = 0; i < 9; i++) {
            for (int j = 0; j < 9; j++) {
                final Rectangle r = new Rectangle(i * fieldSize, j * fieldSize, fieldSize, fieldSize);
                g.fillRect(r.x + fifth, r.y + fifth, r.width - fifth * 2, r.height - fifth * 2);
            }
        }


        g.setColor(Color.BLACK);
        for (Fence f : b.getFences()) {
            int w, h;
            switch (f.getOrientation()) {
                case HORIZONTAL: w = fieldSize - fifth; h = fifth; break;
                case VERTICAL: w = fifth; h = fieldSize - fifth; break;
                default: throw new IllegalStateException();
            }
            int column = (f.getColumn() - 'A') + 1;
            int row = 9 - f.getRow();
            Rectangle r = new Rectangle(
                column * fieldSize - w,
                row * fieldSize - h,
                2 * w,
                2 * h
            );
            g.fill(r);
        }

        int cnt = 0;
        Color[] colors = { Color.WHITE, Color.BLACK, Color.ORANGE, Color.MAGENTA };
        int diametr = fieldSize - fifth * 4;
        for (Player p : b.getPlayers()) {
            int column = p.getColumn();
            int row = 8 - p.getRow();
            final Rectangle r = new Rectangle(
                column * fieldSize + 2 * fifth,
                row * fieldSize + 2 * fifth,
                diametr,
                diametr
            );
            g.setColor(colors[cnt]);
            g.fillOval(r.x, r.y, r.width, r.height);
            if (p == b.getCurrentPlayer()) {
                g.setColor(Color.lightGray);
                g.fillOval(r.x + r.width / 3, r.y + r.height / 3, r.width / 3, r.height / 3);
            }
            cnt++;
        }

        g.setColor(Color.BLACK);
        final Font f = new Font("Monospace", Font.BOLD, fifth * 2);
        g.setFont(f);
        for (int i = 0; i < 8; i++) {
            final char ch = (char) ('A' + i);
            g.drawString(Character.toString(ch), i * fieldSize + fieldSize - fifth, fifth + f.getSize() - f.getBaselineFor(ch));
        }
        for (int i = 0; i < 8; i++) {
            String s = "" + (8 - i);
            g.drawString(s, fifth, i * fieldSize + fieldSize - fifth + f.getSize() - f.getBaselineFor(s.charAt(0)));
        }

        return img;
    }
}
