/*
 * Quoridor server and related libraries
 * Copyright (C) 2009-2010 Jaroslav Tulach <jaroslav.tulach@apidesign.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. Look for COPYING file in the top folder.
 * If not, see http://www.gnu.org/licenses/.
 */

package cz.xelfi.quoridor.webidor;

import java.util.Comparator;
import java.util.Date;
import java.util.UUID;
import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlAttribute;
import javax.xml.bind.annotation.XmlID;
import javax.xml.bind.annotation.XmlRootElement;

/** Basic identification of a game.
 *
 * @author Jaroslav Tulach <jtulach@netbeans.org>
 */
@XmlRootElement
@XmlAccessorType(XmlAccessType.FIELD)
public class GameId {
    public static final Comparator<GameId> NEWEST_FIRST = new NewestFirst();

    @XmlAttribute
    private String white;
    @XmlAttribute
    private String black;
    @XmlAttribute
    private long started;
    @XmlAttribute
    private long modified;
    @XmlAttribute
    private GameStatus status;
    @XmlID @XmlAttribute
    private String id;
    @XmlAttribute
    private int comments;
    @XmlAttribute
    private boolean finished;

    GameId() {
    }

    public GameId(String first, String second) {
        this(first, second, new Date());
    }
    private GameId(String first, String second, Date d) {
        this(
            UUID.randomUUID().toString(),
            first, second, d, d, GameStatus.whiteMove, 0, false
        );
    }

    public GameId(
        String id, String first, String second,
        Date started, Date last, GameStatus result,
        int comments, boolean finished
    ) {
        this.white = first;
        this.black = second;
        this.id = id;
        this.started = started.getTime();
        this.modified = last.getTime();
        this.status = result;
        this.comments = comments;
        this.finished = finished;
    }

    public String getId() {
        return id;
    }

    public String getWhite() {
        return white;
    }

    public String getBlack() {
        return black;
    }

    public long getStarted() {
        return started;
    }

    public long getModified() {
        return modified;
    }

    public GameStatus getStatus() {
        return status;
    }

    public int getComments() {
        return comments;
    }

    public boolean isFinished() {
        return finished;
    }

    private static final class NewestFirst implements Comparator<GameId> {
        public int compare(GameId o1, GameId o2) {
            if (o1 == o2) {
                return 0;
            }
            long diff = o2.getModified() - o1.getModified();
            if (diff != 0) {
                return diff < 0 ? -1 : 1;
            }
            return o1.getId().compareTo(o2.getId());
        }
    }
}
